@@ -9,19 +9,27 @@ from enum import IntFlag
 from collections import defaultdict
 from typing import Optional, Tuple, Sequence, Dict, Any, List, Union
 from abc import ABC, abstractmethod
+import time
 
+from ._kmer_finder import KmerFinder
 from .align import (
     EndSkip,
     Aligner,
     PrefixComparer,
     SuffixComparer,
     edit_environment,
-    hamming_environment,
+    hamming_sphere,
 )
+from .kmer_heuristic import create_positions_and_kmers, kmer_probability_analysis
 
 logger = logging.getLogger()
 
 
+class MockKmerFinder:
+    def kmers_present(self, sequence: str):
+        return True
+
+
 class InvalidCharacter(Exception):
     pass
 
@@ -532,8 +540,8 @@ class SingleAdapter(Adapter, ABC):
         The error rate is the number of errors in the alignment divided by the length
         of the part of the alignment that matches the adapter.
 
-        min_overlap: Report a match only if at least this number of bases of the adapter match the
-            read.
+        min_overlap: Report a match only if at least this number of bases of the adapter are
+            aligned to the read.
 
         read_wildcards: Whether IUPAC wildcards in the read are allowed.
 
@@ -582,14 +590,15 @@ class SingleAdapter(Adapter, ABC):
         self.read_wildcards: bool = read_wildcards
         self.indels: bool = indels
         self.aligner = self._aligner()
+        self.kmer_finder = self._kmer_finder()
 
-    def _make_aligner(self, flags: int) -> Aligner:
+    def _make_aligner(self, sequence: str, flags: int) -> Aligner:
         # TODO
         # Indels are suppressed by setting their cost very high, but a different algorithm
         # should be used instead.
         indel_cost = 1 if self.indels else 100000
         return Aligner(
-            self.sequence,
+            sequence,
             self.max_error_rate,
             flags=flags,
             wildcard_ref=self.adapter_wildcards,
@@ -598,6 +607,27 @@ class SingleAdapter(Adapter, ABC):
             min_overlap=self.min_overlap,
         )
 
+    def _make_kmer_finder(
+        self,
+        sequence: str,
+        back_adapter: bool,
+        front_adapter: bool,
+        internal: bool = True,
+    ) -> KmerFinder:
+        positions_and_kmers = create_positions_and_kmers(
+            sequence,
+            self.min_overlap,
+            self.max_error_rate,
+            back_adapter,
+            front_adapter,
+            internal,
+        )
+        if self._debug:
+            print(kmer_probability_analysis(positions_and_kmers))
+        return KmerFinder(
+            positions_and_kmers, self.adapter_wildcards, self.read_wildcards
+        )
+
     def __repr__(self):
         return (
             "<{cls}(name={name!r}, sequence={sequence!r}, "
@@ -623,6 +653,10 @@ class SingleAdapter(Adapter, ABC):
     def _aligner(self):
         pass
 
+    @abstractmethod
+    def _kmer_finder(self):
+        pass
+
     @abstractmethod
     def match_to(self, sequence: str):
         """
@@ -651,7 +685,13 @@ class FrontAdapter(SingleAdapter):
 
     def _aligner(self) -> Aligner:
         return self._make_aligner(
-            Where.ANYWHERE.value if self._force_anywhere else Where.FRONT.value
+            self.sequence,
+            Where.ANYWHERE.value if self._force_anywhere else Where.FRONT.value,
+        )
+
+    def _kmer_finder(self):
+        return self._make_kmer_finder(
+            self.sequence, back_adapter=self._force_anywhere, front_adapter=True
         )
 
     def match_to(self, sequence: str):
@@ -662,6 +702,8 @@ class FrontAdapter(SingleAdapter):
         return None if no match was found given the matching criteria (minimum
         overlap length, maximum error rate).
         """
+        if not self.kmer_finder.kmers_present(sequence):
+            return None
         alignment: Optional[Tuple[int, int, int, int, int, int]] = self.aligner.locate(
             sequence
         )
@@ -691,13 +733,18 @@ class RightmostFrontAdapter(FrontAdapter):
         return "rightmost_five_prime"
 
     def _aligner(self) -> Aligner:
-        self.sequence = self.sequence[::-1]
         aligner = self._make_aligner(
-            Where.ANYWHERE.value if self._force_anywhere else Where.FRONT.value
+            self.sequence[::-1],
+            Where.ANYWHERE.value if self._force_anywhere else Where.BACK.value,
         )
-        self.sequence = self.sequence[::-1]
         return aligner
 
+    def _kmer_finder(self):
+        kmer_finder = self._make_kmer_finder(
+            self.sequence[::-1], back_adapter=True, front_adapter=self._force_anywhere
+        )
+        return kmer_finder
+
     def match_to(self, sequence: str):
         """
         Attempt to match this adapter to the given read.
@@ -706,8 +753,11 @@ class RightmostFrontAdapter(FrontAdapter):
         return None if no match was found given the matching criteria (minimum
         overlap length, maximum error rate).
         """
+        reversed_sequence = sequence[::-1]
+        if not self.kmer_finder.kmers_present(reversed_sequence):
+            return None
         alignment: Optional[Tuple[int, int, int, int, int, int]] = self.aligner.locate(
-            sequence[::-1]
+            reversed_sequence
         )
         if self._debug:
             print(self.aligner.dpmatrix)
@@ -743,7 +793,13 @@ class BackAdapter(SingleAdapter):
 
     def _aligner(self):
         return self._make_aligner(
-            Where.ANYWHERE.value if self._force_anywhere else Where.BACK.value
+            self.sequence,
+            Where.ANYWHERE.value if self._force_anywhere else Where.BACK.value,
+        )
+
+    def _kmer_finder(self):
+        return self._make_kmer_finder(
+            self.sequence, back_adapter=True, front_adapter=self._force_anywhere
         )
 
     def match_to(self, sequence: str):
@@ -754,6 +810,8 @@ class BackAdapter(SingleAdapter):
         return None if no match was found given the matching criteria (minimum
         overlap length, maximum error rate).
         """
+        if not self.kmer_finder.kmers_present(sequence):
+            return None
         alignment: Optional[Tuple[int, int, int, int, int, int]] = self.aligner.locate(
             sequence
         )
@@ -782,7 +840,12 @@ class AnywhereAdapter(SingleAdapter):
         return "anywhere"
 
     def _aligner(self):
-        return self._make_aligner(Where.ANYWHERE.value)
+        return self._make_aligner(self.sequence, Where.ANYWHERE.value)
+
+    def _kmer_finder(self):
+        return self._make_kmer_finder(
+            self.sequence, back_adapter=True, front_adapter=True
+        )
 
     def match_to(self, sequence: str):
         """
@@ -792,6 +855,8 @@ class AnywhereAdapter(SingleAdapter):
         return None if no match was found given the matching criteria (minimum
         overlap length, maximum error rate).
         """
+        if not self.kmer_finder.kmers_present(sequence):
+            return None
         alignment = self.aligner.locate(sequence.upper())
         if self._debug:
             print(self.aligner.dpmatrix)
@@ -820,9 +885,19 @@ class NonInternalFrontAdapter(FrontAdapter):
         return "noninternal_five_prime"
 
     def _aligner(self):
-        return self._make_aligner(Where.FRONT_NOT_INTERNAL.value)
+        return self._make_aligner(self.sequence, Where.FRONT_NOT_INTERNAL.value)
+
+    def _kmer_finder(self):
+        return self._make_kmer_finder(
+            self.sequence,
+            front_adapter=True,
+            back_adapter=self._force_anywhere,
+            internal=False,
+        )
 
     def match_to(self, sequence: str):
+        if not self.kmer_finder.kmers_present(sequence):
+            return None
         # The locate function takes care of uppercasing the sequence
         alignment = self.aligner.locate(sequence)
         if self._debug:
@@ -847,9 +922,19 @@ class NonInternalBackAdapter(BackAdapter):
         return "noninternal_three_prime"
 
     def _aligner(self):
-        return self._make_aligner(Where.BACK_NOT_INTERNAL.value)
+        return self._make_aligner(self.sequence, Where.BACK_NOT_INTERNAL.value)
+
+    def _kmer_finder(self):
+        return self._make_kmer_finder(
+            self.sequence,
+            back_adapter=True,
+            front_adapter=self._force_anywhere,
+            internal=False,
+        )
 
     def match_to(self, sequence: str):
+        if not self.kmer_finder.kmers_present(sequence):
+            return None
         # The locate function takes care of uppercasing the sequence
         alignment = self.aligner.locate(sequence)
         if self._debug:
@@ -888,7 +973,15 @@ class PrefixAdapter(NonInternalFrontAdapter):
                 min_overlap=self.min_overlap,
             )
         else:
-            return self._make_aligner(Where.PREFIX.value)
+            return self._make_aligner(self.sequence, Where.PREFIX.value)
+
+    def _kmer_finder(self):
+        if isinstance(self.aligner, PrefixComparer):
+            # Prefix comparer does not create a dynamic programming matrix
+            # so the heuristic will be slow and unnecessary.
+            return MockKmerFinder()
+        else:
+            return super()._kmer_finder()
 
     def spec(self) -> str:
         return f"^{self.sequence}..."
@@ -917,7 +1010,15 @@ class SuffixAdapter(NonInternalBackAdapter):
                 min_overlap=self.min_overlap,
             )
         else:
-            return self._make_aligner(Where.SUFFIX.value)
+            return self._make_aligner(self.sequence, Where.SUFFIX.value)
+
+    def _kmer_finder(self):
+        if isinstance(self.aligner, SuffixComparer):
+            # Suffix comparer does not create a dynamic programming matrix
+            # so the heuristic will be slow and unnecessary.
+            return MockKmerFinder()
+        else:
+            return super()._kmer_finder()
 
     def spec(self) -> str:
         return f"{self.sequence}$"
@@ -1193,6 +1294,7 @@ class IndexedAdapters(Matchable, ABC):
         return True
 
     def _make_index(self) -> Tuple[List[int], "AdapterIndex"]:
+        start_time = time.time()
         logger.info("Building index of %s adapters ...", len(self._adapters))
         index: Dict[str, Tuple[SingleAdapter, int, int]] = dict()
         lengths = set()
@@ -1200,46 +1302,74 @@ class IndexedAdapters(Matchable, ABC):
         for adapter in self._adapters:
             sequence = adapter.sequence
             k = int(adapter.max_error_rate * len(sequence))
-            environment = edit_environment if adapter.indels else hamming_environment
-            for s, errors, matches in environment(sequence, k):
-                if s in index:
-                    other_adapter, other_errors, other_matches = index[s]
-                    if matches < other_matches:
-                        continue
-                    if other_matches == matches and not has_warned:
-                        logger.warning(
-                            "Adapters %s %r and %s %r are very similar. At %s allowed errors, "
-                            "the sequence %r cannot be assigned uniquely because the number of "
-                            "matches is %s compared to both adapters.",
-                            other_adapter.name,
-                            other_adapter.sequence,
-                            adapter.name,
-                            adapter.sequence,
-                            k,
-                            s,
-                            matches,
-                        )
-                        has_warned = True
-                else:
-                    index[s] = (adapter, errors, matches)
-                lengths.add(len(s))
-        logger.info("Built an index containing %s strings.", len(index))
+
+            if adapter.indels:
+                for s, errors, matches in edit_environment(sequence, k):
+                    if s in index:
+                        other_adapter, other_errors, other_matches = index[s]
+                        if matches < other_matches:
+                            continue
+                        if other_matches == matches and not has_warned:
+                            self._warn_similar(adapter, other_adapter, k, s, matches)
+                            has_warned = True
+                    else:
+                        index[s] = (adapter, errors, matches)
+                    lengths.add(len(s))
+            else:
+                n = len(sequence)
+                for errors in range(k + 1):
+                    for s in hamming_sphere(sequence, errors):
+                        matches = n - errors
+                        if s in index:
+                            other_adapter, other_errors, other_matches = index[s]
+                            if matches < other_matches:
+                                continue
+                            if other_matches == matches and not has_warned:
+                                self._warn_similar(
+                                    adapter, other_adapter, k, s, matches
+                                )
+                                has_warned = True
+                        else:
+                            index[s] = (adapter, errors, matches)
+                lengths.add(n)
+        elapsed = time.time() - start_time
+        logger.info(
+            "Built an index containing %s strings in %.1f s.", len(index), elapsed
+        )
 
         return sorted(lengths, reverse=True), index
 
+    @staticmethod
+    def _warn_similar(adapter, other_adapter, k, s, matches):
+        logger.warning(
+            "Adapters %s %r and %s %r are very similar. At %s allowed errors, "
+            "the sequence %r cannot be assigned uniquely because the number of "
+            "matches is %s compared to both adapters.",
+            other_adapter.name,
+            other_adapter.sequence,
+            adapter.name,
+            adapter.sequence,
+            k,
+            s,
+            matches,
+        )
+
     def _match_to_one_length(self, sequence: str):
         """
-        Match the adapters against a string and return a Match that represents
+        Match a query string against all adapters and return a Match that represents
         the best match or None if no match was found
         """
         affix = self._make_affix(sequence.upper(), self._length)
         if "N" in affix:
-            # Fall back to non-indexed matching
-            return self._multiple_adapters.match_to(sequence)
-        try:
-            adapter, e, m = self._index[affix]
-        except KeyError:
-            return None
+            result = self._lookup_with_n(affix)
+            if result is None:
+                return None
+            adapter, e, m = result
+        else:
+            try:
+                adapter, e, m = self._index[affix]
+            except KeyError:
+                return None
         return self._make_match(adapter, self._length, m, e, sequence)
 
     def _match_to_multiple_lengths(self, sequence: str):
@@ -1254,20 +1384,24 @@ class IndexedAdapters(Matchable, ABC):
         best_length = 0
         best_m = -1
         best_e = 1000
-        check_n = True
+
+        # Check successively shorter affixes
         for length in self._lengths:
             if length < best_m:
                 # No chance of getting the same or a higher number of matches, so we can stop early
                 break
             affix = self._make_affix(affix, length)
-            if check_n:
-                if "N" in affix:
-                    return self._multiple_adapters.match_to(sequence)
-                check_n = False
-            try:
-                adapter, e, m = self._index[affix]
-            except KeyError:
-                continue
+            if "N" in affix:
+                result = self._lookup_with_n(affix)
+                if result is None:
+                    continue
+                adapter, e, m = result
+            else:
+                try:
+                    adapter, e, m = self._index[affix]
+                except KeyError:
+                    continue
+
             if m > best_m or (m == best_m and e < best_e):
                 # TODO this could be made to work:
                 # assert best_m == -1
@@ -1281,6 +1415,24 @@ class IndexedAdapters(Matchable, ABC):
         else:
             return self._make_match(best_adapter, best_length, best_m, best_e, sequence)
 
+    def _lookup_with_n(self, affix):
+        # N wildcards need to be counted as mismatches (read wildcards aren’t allowed).
+        # We can thus look up an affix where we replace N with an arbitrary nucleotide.
+        affix_without_n = affix.replace("N", "A")
+        try:
+            result = self._index[affix_without_n]
+        except KeyError:
+            return None
+
+        # The looked up number of matches and errors is too low if
+        # the adapter actually has an A where the N is in the query.
+        # Fix this by re-doing the alignment.
+        adapter = result[0]
+        match = adapter.match_to(affix)
+        if match is None:
+            return None
+        return adapter, match.errors, match.score
+
     def enable_debug(self):
         pass
 
