@@ -11,7 +11,7 @@ from abc import ABC, abstractmethod
 
 from dnaio import record_names_match, SequenceRecord
 
-from .qualtrim import quality_trim_index, nextseq_trim_index
+from .qualtrim import quality_trim_index, nextseq_trim_index, poly_a_trim_index
 from .adapters import (
     MultipleAdapters,
     SingleAdapter,
@@ -520,6 +520,7 @@ class Renamer(SingleEndModifier):
     }
 
     def __init__(self, template: str):
+        template = template.replace(r"\t", "\t")
         try:
             self._tokens = list(tokenize_braces(template))
         except TokenizeError as e:
@@ -608,7 +609,7 @@ class PairedEndRenamer(PairedEndModifier):
         except TokenizeError as e:
             raise InvalidTemplate(f"Error in template '{template}': {e}")
         Renamer.raise_if_invalid_variable(self._tokens, self._get_allowed_variables())
-        self._template = template
+        self._template = template.replace(r"\t", "\t")
 
     @staticmethod
     def _get_allowed_variables() -> Set[str]:
@@ -751,6 +752,19 @@ class QualityTrimmer(SingleEndModifier):
         return read[start:stop]
 
 
+class PolyATrimmer(SingleEndModifier):
+    def __init__(self):
+        self.trimmed_bases = 0
+
+    def __repr__(self):
+        return "PolyATrimmer()"
+
+    def __call__(self, record: SequenceRecord, info: ModificationInfo):
+        index = poly_a_trim_index(record.sequence)
+        self.trimmed_bases += len(record) - index
+        return record[:index]
+
+
 class Shortener(SingleEndModifier):
     """Unconditionally shorten a read to the given length
 
