@@ -28,9 +28,10 @@ from .modifiers import (
     PairedAdapterCutter,
     ReverseComplementer,
     PairedEndModifierWrapper,
+    PolyATrimmer,
 )
 from .statistics import ReadLengthStatistics
-from .steps import SingleEndFilter, PairedEndFilter, HasStatistics
+from .steps import HasStatistics, HasFilterStatistics
 
 FILTERS = {
     "too_short": "that were too short",
@@ -70,6 +71,7 @@ class Statistics:
         self.read_length_statistics = ReadLengthStatistics()
         self.with_adapters: List[Optional[int]] = [None, None]
         self.quality_trimmed_bp: List[Optional[int]] = [None, None]
+        self.poly_a_trimmed_bp: List[Optional[int]] = [None, None]
         self.adapter_stats: List[List[AdapterStatistics]] = [[], []]
         self._collected: bool = False
 
@@ -141,9 +143,9 @@ class Statistics:
     def _collect_step(self, step) -> None:
         if isinstance(step, HasStatistics):
             self.read_length_statistics += step.get_statistics()
-        if isinstance(step, (SingleEndFilter, PairedEndFilter)):
+        if isinstance(step, HasFilterStatistics):
             name = step.descriptive_identifier()
-            self.filtered[name] = step.filtered
+            self.filtered[name] = step.filtered()
 
     def _collect_modifier(self, m) -> None:
         if isinstance(m, PairedAdapterCutter):
@@ -160,6 +162,10 @@ class Statistics:
                 self.quality_trimmed_bp[i] = add_if_not_none(
                     self.quality_trimmed_bp[i], modifier.trimmed_bases
                 )
+            if isinstance(modifier, PolyATrimmer):
+                self.poly_a_trimmed_bp[i] = add_if_not_none(
+                    self.poly_a_trimmed_bp[i], modifier.trimmed_bases
+                )
             elif isinstance(modifier, AdapterCutter):
                 assert self.with_adapters[i] is None
                 self.with_adapters[i] = modifier.with_adapters
@@ -201,6 +207,9 @@ class Statistics:
                 "quality_trimmed": self.quality_trimmed,
                 "quality_trimmed_read1": self.quality_trimmed_bp[0],
                 "quality_trimmed_read2": self.quality_trimmed_bp[1],
+                "poly_a_trimmed": self.poly_a_trimmed,
+                "poly_a_trimmed_read1": self.poly_a_trimmed_bp[0],
+                "poly_a_trimmed_read2": self.poly_a_trimmed_bp[1],
                 "output": self.total_written_bp,
                 "output_read1": written_bp[0],
                 "output_read2": written_bp[1] if self.paired else None,
@@ -292,6 +301,10 @@ class Statistics:
     def quality_trimmed(self) -> Optional[int]:
         return add_if_not_none(*self.quality_trimmed_bp)
 
+    @property
+    def poly_a_trimmed(self) -> Optional[int]:
+        return add_if_not_none(*self.poly_a_trimmed_bp)
+
     @property
     def total_written_bp(self) -> int:
         return sum(self.read_length_statistics.written_bp())
@@ -327,6 +340,10 @@ class Statistics:
     def filtered_fraction(self, filter_name: str) -> float:
         return safe_divide(self.filtered.get(filter_name), self.n)
 
+    @property
+    def poly_a_trimmed_fraction(self) -> float:
+        return safe_divide(self.poly_a_trimmed, self.total)
+
 
 class ErrorRanges:
     """
@@ -608,6 +625,16 @@ def full_report(stats: Statistics, time: float, gc_content: float) -> str:  # no
                 if stats.quality_trimmed_bp[i] is not None:
                     report += f"  Read {i + 1}: {stats.quality_trimmed_bp[i]:13,d} bp\n"
 
+    if stats.poly_a_trimmed is not None:
+        report += (
+            "Poly-A-trimmed:            "
+            f"{stats.poly_a_trimmed:13,d} bp ({stats.poly_a_trimmed_fraction:.1%})\n"
+        )
+        if stats.paired:
+            for i in (0, 1):
+                if stats.poly_a_trimmed_bp[i] is not None:
+                    report += f"  Read {i + 1}: {stats.poly_a_trimmed_bp[i]:13,d} bp\n"
+
     report += (
         "Total written (filtered):  "
         "{o.total_written_bp:13,d} bp ({o.total_written_bp_fraction:.1%})\n"
