@@ -6,7 +6,7 @@ from io import StringIO, BytesIO
 import dnaio
 import pytest
 
-from cutadapt.__main__ import main
+from cutadapt.cli import main
 from utils import assert_files_equal, datapath, cutpath
 
 # pytest.mark.timeout will not fail even if pytest-timeout is not installed
@@ -264,7 +264,7 @@ def test_twoadapters(run):
     run("-a AATTTCAGGAATT -a GTTCTCTAGTTCT", "twoadapters.fasta", "twoadapters.fasta")
 
 
-def test_polya(run):
+def test_poly_a_legacy(run):
     """poly-A tails"""
     run(
         "-m 24 -O 10 -a AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
@@ -273,11 +273,15 @@ def test_polya(run):
     )
 
 
-def test_polya_brace_notation(run):
+def test_poly_a_legacy_brace_notation(run):
     """poly-A tails"""
     run("-m 24 -O 10 -a A{35}", "polya.fasta", "polya.fasta")
 
 
+def test_poly_a(run):
+    run("--poly-a", "poly-a.fasta", "polya.fasta")
+
+
 # the same as --action=none
 def test_no_trim(run):
     run("--no-trim --discard-untrimmed -a CCCTAGTTAAAC", "no-trim.fastq", "small.fastq")
@@ -1105,3 +1109,20 @@ def test_terminates_correctly_on_error_in_subprocess(tmp_path):
     ]
     with pytest.raises(SystemExit):
         main(params)
+
+
+def test_json_report_with_demultiplexing_and_discard_untrimmed(tmp_path):
+    stats = main(
+        [
+            "--json",
+            str(tmp_path / "demux.cutadapt.json"),
+            "--discard-untrimmed",
+            "-a",
+            "name=ACGT",
+            "-o",
+            str(tmp_path / "{name}.fastq"),
+            datapath("illumina.fastq.gz"),
+        ]
+    )
+    assert stats.n == 100
+    assert stats.written == 64
