@@ -1,7 +1,7 @@
 from typing import List
 
 import pytest
-from dnaio import Sequence
+from dnaio import SequenceRecord
 from cutadapt.adapters import (
     BackAdapter,
     PrefixAdapter,
@@ -31,7 +31,7 @@ from cutadapt.modifiers import (
 
 def test_unconditional_cutter():
     UnconditionalCutter(length=5)
-    read = Sequence("r1", "abcdefg")
+    read = SequenceRecord("r1", "abcdefg")
 
     info = ModificationInfo(read)
     assert UnconditionalCutter(length=2)(read, info).sequence == "cdefg"
@@ -55,13 +55,13 @@ def test_reverse_complementer():
     adapter_cutter = AdapterCutter(adapters, index=False)
     reverse_complementer = ReverseComplementer(adapter_cutter)
 
-    read = Sequence("r", "ttatttgtctCCAGCTTAGACATATCGCCT")
+    read = SequenceRecord("r", "ttatttgtctCCAGCTTAGACATATCGCCT")
     info = ModificationInfo(read)
     trimmed = reverse_complementer(read, info)
     assert trimmed.sequence == "CCAGCTTAGACATATCGCCT"
     assert not info.is_rc
 
-    read = Sequence("r", "CAACAGGCCACATTAGACATATCGGATGGTagacaaataa")
+    read = SequenceRecord("r", "CAACAGGCCACATTAGACATATCGGATGGTagacaaataa")
     info = ModificationInfo(read)
     trimmed = reverse_complementer(read, info)
     assert trimmed.sequence == "ACCATCCGATATGTCTAATGTGGCCTGTTG"
@@ -70,7 +70,7 @@ def test_reverse_complementer():
 
 def test_zero_capper():
     zc = ZeroCapper()
-    read = Sequence("r1", "ACGT", "# !%")
+    read = SequenceRecord("r1", "ACGT", "# !%")
     result = zc(read, ModificationInfo(read))
     assert result.sequence == "ACGT"
     assert result.qualities == "#!!%"
@@ -81,37 +81,41 @@ def test_nend_trimmer():
     seqs = ["NNNNAAACCTTGGNNN", "NNNNAAACNNNCTTGGNNN", "NNNNNN"]
     trims = ["AAACCTTGG", "AAACNNNCTTGG", ""]
     for seq, trimmed in zip(seqs, trims):
-        _seq = Sequence("read1", seq, qualities="#" * len(seq))
-        _trimmed = Sequence("read1", trimmed, qualities="#" * len(trimmed))
+        _seq = SequenceRecord("read1", seq, qualities="#" * len(seq))
+        _trimmed = SequenceRecord("read1", trimmed, qualities="#" * len(trimmed))
         assert trimmer(_seq, ModificationInfo(_seq)) == _trimmed
 
 
 def test_quality_trimmer():
-    read = Sequence("read1", "ACGTTTACGTA", "##456789###")
+    read = SequenceRecord("read1", "ACGTTTACGTA", "##456789###")
 
     qt = QualityTrimmer(10, 10, 33)
-    assert qt(read, ModificationInfo(read)) == Sequence("read1", "GTTTAC", "456789")
+    assert qt(read, ModificationInfo(read)) == SequenceRecord(
+        "read1", "GTTTAC", "456789"
+    )
 
     qt = QualityTrimmer(0, 10, 33)
-    assert qt(read, ModificationInfo(read)) == Sequence("read1", "ACGTTTAC", "##456789")
+    assert qt(read, ModificationInfo(read)) == SequenceRecord(
+        "read1", "ACGTTTAC", "##456789"
+    )
 
     qt = QualityTrimmer(10, 0, 33)
-    assert qt(read, ModificationInfo(read)) == Sequence(
+    assert qt(read, ModificationInfo(read)) == SequenceRecord(
         "read1", "GTTTACGTA", "456789###"
     )
 
 
 def test_shortener():
-    read = Sequence("read1", "ACGTTTACGTA", "##456789###")
+    read = SequenceRecord("read1", "ACGTTTACGTA", "##456789###")
 
     shortener = Shortener(0)
-    assert shortener(read, ModificationInfo(read)) == Sequence("read1", "", "")
+    assert shortener(read, ModificationInfo(read)) == SequenceRecord("read1", "", "")
 
     shortener = Shortener(1)
-    assert shortener(read, ModificationInfo(read)) == Sequence("read1", "A", "#")
+    assert shortener(read, ModificationInfo(read)) == SequenceRecord("read1", "A", "#")
 
     shortener = Shortener(5)
-    assert shortener(read, ModificationInfo(read)) == Sequence(
+    assert shortener(read, ModificationInfo(read)) == SequenceRecord(
         "read1", "ACGTT", "##456"
     )
 
@@ -146,8 +150,8 @@ class TestPairedAdapterCutter:
     def test_actions(self, action, expected_trimmed1, expected_trimmed2):
         a1 = BackAdapter("GGTTAA")
         a2 = BackAdapter("AACCGG")
-        s1 = Sequence("name", "CCCCGGTTAACCCC")
-        s2 = Sequence("name", "TTTTAACCGGTTTT")
+        s1 = SequenceRecord("name", "CCCCGGTTAACCCC")
+        s2 = SequenceRecord("name", "TTTTAACCGGTTTT")
         pac = PairedAdapterCutter([a1], [a2], action=action)
         info1 = ModificationInfo(s1)
         info2 = ModificationInfo(s2)
@@ -160,8 +164,8 @@ class TestPairedAdapterCutter:
         r1_a2 = BackAdapter("CCCC")
         r2_a1 = BackAdapter("GGGG")
         r2_a2 = BackAdapter("TTTT")
-        s1 = Sequence("name", "TTAAAAAATTCCCCTT")
-        s2 = Sequence("name", "ACACTTTTACAC")
+        s1 = SequenceRecord("name", "TTAAAAAATTCCCCTT")
+        s2 = SequenceRecord("name", "ACACTTTTACAC")
         pac = PairedAdapterCutter([r1_a1, r1_a2], [r2_a1, r2_a2], action="lowercase")
         info1 = ModificationInfo(s1)
         info2 = ModificationInfo(s2)
@@ -181,7 +185,7 @@ def test_retain_times():
 def test_action_retain():
     back = BackAdapter("AACCGG")
     ac = AdapterCutter([back], action="retain")
-    seq = Sequence("r1", "ATTGCCAACCGGTATATAT")
+    seq = SequenceRecord("r1", "ATTGCCAACCGGTATATAT")
     info = ModificationInfo(seq)
     trimmed = ac(seq, info)
     assert "ATTGCCAACCGG" == trimmed.sequence
@@ -205,7 +209,7 @@ def test_linked_action_retain(s, expected):
         )
     ]
     ac = AdapterCutter(adapters, action="retain")
-    seq = Sequence("r1", s)
+    seq = SequenceRecord("r1", s)
     info = ModificationInfo(seq)
     trimmed = ac(seq, info)
     assert expected == trimmed.sequence
@@ -218,55 +222,61 @@ class TestRenamer:
 
     def test_header_template_variable(self):
         renamer = Renamer("{header} extra")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         assert renamer(read, info).name == "theid thecomment extra"
 
     def test_id_template_variable(self):
         renamer = Renamer("{id} extra")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         assert renamer(read, info).name == "theid extra"
 
+    def test_tab_escape(self):
+        renamer = Renamer(r"{id} extra\tand a tab")
+        read = SequenceRecord("theid thecomment", "ACGT")
+        info = ModificationInfo(read)
+        assert renamer(read, info).name == "theid extra\tand a tab"
+
     def test_comment_template_variable(self):
         renamer = Renamer("{id}_extra {comment}")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         assert renamer(read, info).name == "theid_extra thecomment"
 
     def test_comment_template_variable_missing_comment(self):
         renamer = Renamer("{id}_extra {comment}")
-        read = Sequence("theid", "ACGT")
+        read = SequenceRecord("theid", "ACGT")
         info = ModificationInfo(read)
         assert renamer(read, info).name == "theid_extra "
 
     def test_cut_prefix_template_variable(self):
         renamer = Renamer("{id}_{cut_prefix} {comment}")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         info.cut_prefix = "TTAAGG"
         assert renamer(read, info).name == "theid_TTAAGG thecomment"
 
     def test_cut_suffix_template_variable(self):
         renamer = Renamer("{id}_{cut_suffix} {comment}")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         info.cut_suffix = "TTAAGG"
         assert renamer(read, info).name == "theid_TTAAGG thecomment"
 
     def test_rc_template_variable(self):
         renamer = Renamer("{id} rc={rc} {comment}")
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info = ModificationInfo(read)
         assert renamer(read, info).name == "theid rc= thecomment"
 
-        read = Sequence("theid thecomment", "ACGT")
+        read = SequenceRecord("theid thecomment", "ACGT")
         info.is_rc = True
         assert renamer(read, info).name == "theid rc=rc thecomment"
 
     def test_match_sequence(self):
         sequence = "TTTTCCCCACGTGGGG"
-        read = Sequence("theid thecomment", sequence)
+        read = SequenceRecord("theid thecomment", sequence)
         adapter = BackAdapter("AGGT")
         info = ModificationInfo(read)
         info.matches.append(
@@ -289,7 +299,7 @@ class TestRenamer:
 
     def test_match_sequence_linked_match(self):
         sequence = "TATTCCCCACGTGGGG"
-        read = Sequence("theid thecomment", sequence)
+        read = SequenceRecord("theid thecomment", sequence)
         adapter1 = PrefixAdapter("TTTT")
         adapter2 = BackAdapter("AGGT")
         linked_adapter = LinkedAdapter(
@@ -333,10 +343,20 @@ class TestPairedEndRenamer:
         with pytest.raises(InvalidTemplate):
             PairedEndRenamer("{id} {invalid}")
 
+    def test_tab_escape(self):
+        renamer = PairedEndRenamer(r"{id} {comment}\tand a tab")
+        r1 = SequenceRecord("theid comment1", "ACGT")
+        r2 = SequenceRecord("theid comment2", "ACGT")
+        info1 = ModificationInfo(r1)
+        info2 = ModificationInfo(r2)
+        renamed1, renamed2 = renamer(r1, r2, info1, info2)
+        assert renamed1.name == "theid comment1\tand a tab"
+        assert renamed2.name == "theid comment2\tand a tab"
+
     def test_ids_not_identical(self):
         renamer = PairedEndRenamer("{id} abc {comment} xyz")
-        r1 = Sequence("theid_a cmtx", "ACGT")
-        r2 = Sequence("theid_b cmty", "ACGT")
+        r1 = SequenceRecord("theid_a cmtx", "ACGT")
+        r2 = SequenceRecord("theid_b cmty", "ACGT")
         info1 = ModificationInfo(r1)
         info2 = ModificationInfo(r2)
         with pytest.raises(ValueError) as e:
@@ -345,8 +365,8 @@ class TestPairedEndRenamer:
 
     def test_comment(self):
         renamer = PairedEndRenamer("{id} abc {comment} xyz")
-        r1 = Sequence("theid cmtx", "ACGT")
-        r2 = Sequence("theid cmty", "ACGT")
+        r1 = SequenceRecord("theid cmtx", "ACGT")
+        r2 = SequenceRecord("theid cmty", "ACGT")
         info1 = ModificationInfo(r1)
         info2 = ModificationInfo(r2)
         renamed1, renamed2 = renamer(r1, r2, info1, info2)
@@ -355,8 +375,8 @@ class TestPairedEndRenamer:
 
     def test_r1_comment(self):
         renamer = PairedEndRenamer("{id} abc {r1.comment} xyz")
-        r1 = Sequence("theid cmtx", "ACGT")
-        r2 = Sequence("theid cmty", "ACGT")
+        r1 = SequenceRecord("theid cmtx", "ACGT")
+        r2 = SequenceRecord("theid cmty", "ACGT")
         info1 = ModificationInfo(r1)
         info2 = ModificationInfo(r2)
         renamed1, renamed2 = renamer(r1, r2, info1, info2)
@@ -365,8 +385,8 @@ class TestPairedEndRenamer:
 
     def test_r2_comment(self):
         renamer = PairedEndRenamer("{id} abc {r2.comment} xyz")
-        r1 = Sequence("theid cmtx", "ACGT")
-        r2 = Sequence("theid cmty", "ACGT")
+        r1 = SequenceRecord("theid cmtx", "ACGT")
+        r2 = SequenceRecord("theid cmty", "ACGT")
         info1 = ModificationInfo(r1)
         info2 = ModificationInfo(r2)
         renamed1, renamed2 = renamer(r1, r2, info1, info2)
@@ -375,8 +395,8 @@ class TestPairedEndRenamer:
 
     def test_read_number(self):
         renamer = PairedEndRenamer("{id} read no. is: {rn}")
-        r1 = Sequence("theid cmtx", "ACGT")
-        r2 = Sequence("theid cmty", "ACGT")
+        r1 = SequenceRecord("theid cmtx", "ACGT")
+        r2 = SequenceRecord("theid cmty", "ACGT")
         info1 = ModificationInfo(r1)
         info2 = ModificationInfo(r2)
         renamed1, renamed2 = renamer(r1, r2, info1, info2)
@@ -384,7 +404,7 @@ class TestPairedEndRenamer:
         assert renamed2.name == "theid read no. is: 2"
 
     def test_match_sequence(self):
-        r1 = Sequence("theid first", "AACC")
+        r1 = SequenceRecord("theid first", "AACC")
         info1 = ModificationInfo(r1)
         info1.matches.append(
             RemoveBeforeMatch(
@@ -398,7 +418,7 @@ class TestPairedEndRenamer:
                 sequence="AACC",
             )
         )
-        r2 = Sequence("theid second", "GGTT")
+        r2 = SequenceRecord("theid second", "GGTT")
         info2 = ModificationInfo(r2)
         info2.matches.append(
             RemoveBeforeMatch(
