@@ -1,13 +1,13 @@
 import pytest
 
-from dnaio import Sequence
-from cutadapt.qualtrim import nextseq_trim_index, expected_errors
+from dnaio import SequenceRecord
+from cutadapt.qualtrim import nextseq_trim_index, expected_errors, poly_a_trim_index
 
 
 def test_nextseq_trim():
-    s = Sequence("n", "", "")
+    s = SequenceRecord("n", "", "")
     assert nextseq_trim_index(s, cutoff=22) == 0
-    s = Sequence(
+    s = SequenceRecord(
         "n",
         "TCTCGTATGCCGTCTTATGCTTGAAAAAAAAAAGGGGGGGGGGGGGGGGGNNNNNNNNNNNGGNGG",
         "AA//EAEE//A6///E//A//EA/EEEEEEAEA//EEEEEEEEEEEEEEE###########EE#EA",
@@ -15,6 +15,24 @@ def test_nextseq_trim():
     assert nextseq_trim_index(s, cutoff=22) == 33
 
 
+@pytest.mark.parametrize(
+    "sequence,tail",
+    [
+        ("", ""),
+        ("TCAAGAAGTCCTTTACCAGCTTTC", "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"),
+        ("TCAAGAAGTCCTTTACCAGCTTTC", "AAATAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"),
+        ("GCAGATCACCTT", "AAAAAAAAAAAAAAAAAAAAAAAAAAAATAAA"),
+        ("GCAGATCACCTT", "AAAAAAAAAAAAAAAAAAAAAAAAAAAAT"),
+        ("GCAGATCACCTT", "AAAAAAAAAAAAAAAAAAAAAAAAAAAATCG"),
+        ("GCAGATCACCTAT", "AAAACAAAAAAACAAAAAAAACAAAAAA"),
+        ("TTTT", "AAATAAAA"),
+        ("GGGGGGGGAAAGAAGAAGAAGAAGAAGAAG", "AAA"),
+    ],
+)
+def test_poly_a_trim_index(sequence, tail):
+    assert poly_a_trim_index(sequence + tail) == len(sequence)
+
+
 def test_expected_errors():
     def encode_qualities(quals):
         return "".join(chr(q + 33) for q in quals)
